﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Emgu.CV.Structure;
using Emgu.CV;
using Emgu.CV.UI;
using System.Drawing;
using System.Runtime.InteropServices;
using Emgu.CV.Util;


namespace TP
{
    class ParamDetecterGraine
    {
        private Image<Bgr, Byte> imageNir;
        Rectangle ROICellule;
        List<Recipient> listeContenant;
        int noCellule;
        typeGraine typeG = new typeGraine();

        /* ----------------------------------------------------------------------------------------------------------
         *  Initialisation de la classe 
         *  ---------------------------------------------------------------------------------------------------------*/


         public ParamDetecterGraine(Image<Bgr, Byte> image, Rectangle roi, List<Recipient> liste, int no, typeGraine t)
         {
             imageNir = image;
             ROICellule = roi;
             listeContenant = liste;
             noCellule = no;
             typeG = t;
         }
		 
		 public ParamDetecterGraine(Image<Bgr, Byte> image, Rectangle roi, Rectangle roiC, List<Recipient> liste, int no, typeGraine t)
         {
             imageNir = image;
             ROICellule = roi;
			 ROIComplet = roiC;
             listeContenant = liste;
             noCellule = no;
             typeG = t;
         }

    
        /* Detection des graines de type t  dans une cellule no de la liste de cellules */


        public void DetecterGrainesCellule()
        {

            listeContenant[noCellule].Moyenne = DetecterMoyenne();

            listeContenant[noCellule].ListeGraine = DetecterGraine(listeContenant[noCellule].Moyenne[1]); /*passe vecteur de moyenne*/
				  
        }

         /* ----------------------------------------------------------------------------------------------------------
          *  Fonction detectant la moyenne de la casette
          *  ---------------------------------------------------------------------------------------------------------*/
         public double[] DetecterMoyenne()
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double moyenne, moyenne_debut, moyenne_fin;
             double somme = 0, somme_debut = 0, somme_fin = 0;

             for (int i = 0; i < imageRGBgray.Height; i++)
                 for (int j = 0; j < imageRGBgray.Width; j++)
                 {
                     if (i < 2) 
                         somme_debut += imageRGBgray.Data[i, j, 0]; /*2 premières rangées*/
                     
                     somme += imageRGBgray.Data[i, j, 0];
                     
                     if (i >= imageRGBgray.Height - 2) 
                         somme_fin += imageRGBgray.Data[i, j, 0]; /*deux dernières rangés*/
                 }
             moyenne_debut = somme_debut /(imageRGBgray.Width * 2);         /*moyenne des pixels au début (2 premières rangées)*/
             moyenne = somme / (imageRGBgray.Height * imageRGBgray.Width);  /*moyenne des pixels au milieu*/
             moyenne_fin = somme_fin / (imageRGBgray.Width * 2);            /*moyenne des pixels à la fin*/

             return new double[3] { moyenne_debut, moyenne, moyenne_fin };
         }

         /* ----------------------------------------------------------------------------------------------------------
         *  Fonction calculant la variance de la casette
         *  ---------------------------------------------------------------------------------------------------------*/
         public double DetecterStd(double moyenne)
         {
             Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();

             double var;
             double somme = 0;

             for (int i = 0; i < imageRGBgray.Height; i++)
                 for (int j = 0; j < imageRGBgray.Width; j++)
                 {
                     somme = somme + ((imageRGBgray.Data[i, j, 0]-moyenne) * (imageRGBgray.Data[i, j, 0]-moyenne));
                     
                 }
             var = somme / (imageRGBgray.Height * imageRGBgray.Width);

             return Math.Sqrt(var);
         }

 
  

 /* ----------------------------------------------------------------------------------------------------------
          *  Fonction detectant la graine
          *  ---------------------------------------------------------------------------------------------------------*/
         public List<Graine> DetecterGraine(double moyenne) /*utilisation de la moyenne pour tresholder*/
         { 
            List<Graine> ListeGraine = new List<Graine>();
            
            Image<Gray, Byte> imageRGBgray = imageNir.Convert<Gray, Byte>().PyrDown().PyrUp();
           double std = DetecterStd(moyenne);
           

           if(moyenne+2.0*std > 255)
              imageRGBgray._ThresholdBinary(new Gray(moyenne + 1.5 * std), new Gray(255));
           else
              imageRGBgray._ThresholdBinary(new Gray(moyenne + 2.0 * std), new Gray(255)); 

             // elemine les spikes
            imageRGBgray.Erode(1).Dilate(1);

            // emplir les trous
           //  imageRGBgray.Dilate(1).Erode(1);

             imageRGBgray.SmoothMedian(7);

             using (MemStorage storage = new MemStorage()) //allocate storage for contour approximation
             
             /*pour chaque contour dans une cellulle*/
             // on detecte les contour selon une grandeur de graine x
             for (Contour<Point> contours = imageRGBgray.FindContours(); contours != null; contours = contours.HNext)
             {
               //  if (contours.Total >= 6 && contours.Area > ((float)typeG.largeurMin * (float)typeG.hauteurMin * Math.PI) && contours.BoundingRectangle.Width > (float)typeG.largeurMin && contours.BoundingRectangle.Width < (float)typeG.largeurMax &&
                //          contours.BoundingRectangle.Height > (float)typeG.hauteurMin && contours.BoundingRectangle.Height < (float)typeG.hauteurMax)
                // if (contours.Total >= 6 && contours.Perimeter > typeG.pmin && contours.Area > typeG.aireMin && contours.Area < (3.0*typeG.aireMax))
                // {

                   // MCvBox2D minR = contours.GetMinAreaRect();

                   // if (contours.Total >= 6 && contours.Perimeter > typeG.pmin && contours.Area > typeG.aireMin && minR.size.Height > typeG.hauteurMin && minR.size.Width > typeG.largeurMin)
                   //{

                   if (contours.Total >= 6 && /*contours.Perimeter > typeG.pmin && */ contours.Area > (typeG.aireMax+typeG.aireMin)/2)
                   {
                     bool ok = true;
                   /*  if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y < 10)
                         ok = false;
                     if (contours.BoundingRectangle.X < 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height -10 )
                         ok = false;
                     if (contours.BoundingRectangle.X + contours.BoundingRectangle.Width > imageRGBgray.Width - 10 && contours.BoundingRectangle.Y + contours.BoundingRectangle.Height > imageRGBgray.Height - 10 )
                         ok = false;

                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + typeG.largeurMax / 2))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - typeG.largeurMax / 2))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) >  (imageRGBgray.Height / 2 + typeG.largeurMax / 2))
                         ok = false;
                    * 
                    */ 
            
            // Elimination du cadre
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) < (imageRGBgray.Width / 2 - 60))
                         ok = false;
                     if ((contours.BoundingRectangle.X + contours.BoundingRectangle.Width / 2) > (imageRGBgray.Width / 2 + 60))
                         ok = false;
                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) < (imageRGBgray.Height / 2 - 60))
                         ok = false;

                     if ((contours.BoundingRectangle.Y + contours.BoundingRectangle.Height / 2) > (imageRGBgray.Height / 2 + 60))
                         ok = false;

            // Elimination des objets longilignes comme des brindilles
                     //if ((contours.GetMinAreaRect().size.Height / contours.GetMinAreaRect().size.Width) > 2.0)
                     //    ok = false;

            // Elimination des objets circulaires associés a la vermiculite

                     //if (typeG.nom == "Épinette blanche")
                     //{
                         if ((contours.Area * 4.0 * Math.PI) / (contours.Perimeter * contours.Perimeter) > .95) // proche d un cercle ?????????
                             ok = false;
                     //}
   
 
                     // si la graine se trouve a 10 pixel des coins du contours // pourquoi
                     if (ok )
                     {
                        
                        Graine gr = new Graine(contours, 1);
                        
                      //  if (gr.Localisation.X > 1  &&gr.Localisation.Y > 1 && imageRGBgray.Width - (gr.Localisation.X + contours.BoundingRectangle.Width) > 1
                       //       && imageRGBgray.Height - (gr.Localisation.Y + contours.BoundingRectangle.Height) > 1)
                      //  {

                        /*http://www.emgu.com/wiki/files/2.0.1.0/html/492603a1-20de-fa97-b128-bd8d55a29e48.htm*/
                /*         Seq<Emgu.CV.Structure.MCvConvexityDefect> te = contours.GetConvexityDefacts(storage, Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);
                            MCvConvexityDefect[] defacts = te.ToArray();
                            List<double> angle = new List<double>();
                            foreach (MCvConvexityDefect d in defacts)
                            {
                                double fA = Math.Sqrt(Math.Pow(d.EndPoint.X - d.DepthPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.DepthPoint.Y, 2)); //distance entre b et c
                                double fB = Math.Sqrt(Math.Pow(d.EndPoint.X - d.StartPoint.X, 2) + Math.Pow(d.EndPoint.Y - d.StartPoint.Y, 2));  //distance entre a et c
                                double fC = Math.Sqrt(Math.Pow(d.DepthPoint.X - d.StartPoint.X, 2) + Math.Pow(d.DepthPoint.Y - d.StartPoint.Y, 2));
                                double fAngle = (Math.Pow(fA, 2) + Math.Pow(fC, 2) - Math.Pow(fB, 2)) / (2 * fA * fC); //(a2 + c2 - b2) / (2*a*c)
                                
                                if (Math.Abs(fAngle) < 0.6) // > 45 < 135 degres
                                    angle.Add(fAngle);
                          
                               // if (Math.Abs(fAngle) < (float)typeG.angle)
                                //    angle.Add(fAngle);

                                //if (Math.Acos(fAngle) < (float)typeG.angle)
                                 //   angle.Add(fAngle);
                               // if (Math.Acos(fAngle) < 135.0)
                                 //   angle.Add(fAngle);

                            }
                  
                 */
                            /*Calcule le nombre de graines à partir des angles */
                            //  System.Console.WriteLine(angle.Count);
                          //  gr.Nb_graine = angle.Count > 0 ? gr.Nb_graine + 1 : gr.Nb_graine;

                            Seq<Point> cHull = contours.GetConvexHull(Emgu.CV.CvEnum.ORIENTATION.CV_CLOCKWISE);

                            gr.Nb_graine = (((float)(contours.Area / cHull.Area) < typeG.solidite) && (contours.Area > (typeG.aireMin * 1.5))) ? gr.Nb_graine + 1 : gr.Nb_graine; 
                           
                            //if((contours.Area/cHull.Area) < 0.9) // solidity 
                          // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < typeG.solidite) && (contours.Area > typeG.aireMax)) ? gr.Nb_graine + 1 : gr.Nb_graine; // 1 graine ou plus que 2

                            //gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.95) && !((contours.Area > typeG.aireMin) && (contours.Area < typeG.aireMax))) ? gr.Nb_graine + 1 : gr.Nb_graine;
                           // gr.Nb_graine = (((float)(contours.Area / cHull.Area) < 0.90) && (contours.Area > 2.0* typeG.aireMin)) ? gr.Nb_graine + 1 : gr.Nb_graine;
                         ListeGraine.Add(gr);
                        }
                   }
                // }
             
             }
             return ListeGraine;
         }

    }
}
